﻿using System;
using System.Data;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;

namespace PureChat.Plugins.OnTime
{
	[ToolboxData("<{0}:HierarchyDropDown runat=server></{0}:HierarchyDropDown>")]
	public sealed class HierarchyDropDown : AxoDropDownList
	{
		#region private members

		private DataTable _dataTableSource;
		private string _dataTextField;
		private string _dataParentField;
		private string _dataIdField;
		private bool _showRootItem;
		private string _rootItemText;
		private int _rootProjectId;
		#endregion

		#region public properties

		public DataTable DataTableSource
		{
			get { return _dataTableSource; }
			set { _dataTableSource = value; }
		}

		public override string DataTextField
		{
			get { return _dataTextField; }
			set { _dataTextField = value; }
		}

		public string DataParentField
		{
			get { return _dataParentField; }
			set { _dataParentField = value; }
		}

		public string DataIdField
		{
			get { return _dataIdField; }
			set { _dataIdField = value; }
		}

		public string RootItemText
		{
			get { return _rootItemText; }
			set { _rootItemText = value; }
		}

		public bool ShowRootItem
		{
			get { return _showRootItem; }
			set { _showRootItem = value; }
		}
		
		public int RootProjectId
		{
			get { return _rootProjectId; }
			set { _rootProjectId = value; }
		}

		#endregion

		#region public methods

		public override void DataBind()
		{
			RefreshHierarchyDropDown();
		}

		public void SelectItemByValue(int itemValue)
		{
			foreach (ListItem item in Items)
			{
				if (int.Parse(item.Value) == itemValue)
				{
					this.SelectedIndex = Items.IndexOf(item);
					break;
				}
			}
		}

		public int GetSelectedItemValue()
		{
			ListItem item = this.SelectedItem;
			if (item == null)
				return 0;
			else
				return Convert.ToInt32(item.Value);
		}

		#endregion

		#region constructor

		public HierarchyDropDown()
		{
		}

		#endregion

		#region private methods

		private void RefreshHierarchyDropDown()
		{
			Items.Clear();

			if (ShowRootItem == true)
			{
				ListItem item = new ListItem();
				item.Text = RootItemText;
				item.Value = 0.ToString();
				Items.Add(item);
			}

			if (DataTableSource != null)
			{
				string sql = string.Empty;
				if (DataParentField != null && DataParentField.Length > 0)
				{
					sql = DataParentField + " = 0";
					if (_rootProjectId > 0)
					{
						sql = DataIdField + " = " + _rootProjectId.ToString();
					}
				}

				foreach (DataRow dr in DataTableSource.Select(sql))
				{
					ListItem item = new ListItem();
					item.Text = dr[DataTextField].ToString();
					item.Value = dr[DataIdField].ToString();

					Items.Add(item);

					if (sql.Length > 0)
					{
						// Add the child nodes of this node
						AddChildItems(item, HttpContext.Current.Server.HtmlDecode("&nbsp;&nbsp;&nbsp;"));
					}
				}
			}
		}

		private void AddChildItems(ListItem parentItem, string indent)
		{
			foreach (DataRow dr in DataTableSource.Select(DataParentField + " = " + parentItem.Value))
			{
				ListItem item = new ListItem();
				item.Text = indent + dr[DataTextField].ToString();
				item.Value = dr[DataIdField].ToString();

				Items.Add(item);

				// Add the child nodes of this node
				AddChildItems(item, indent + HttpContext.Current.Server.HtmlDecode("&nbsp;&nbsp;&nbsp;"));
			}
		}

		#endregion

		#region protected methods

		override protected void RenderContents(HtmlTextWriter writer)
		{
			foreach (ListItem i in Items)
			{
				writer.WriteBeginTag("option");
				if (i.Selected)
					writer.WriteAttribute("selected", "selected", false);
				writer.WriteAttribute("value", i.Value, true);
				IEnumerator d = i.Attributes.Keys.GetEnumerator();
				while (d.MoveNext())
					writer.WriteAttribute(d.Current.ToString(), i.Attributes[d.Current.ToString()]);
				writer.Write('>');
				System.Web.HttpUtility.HtmlEncode(i.Text, writer);
				writer.WriteEndTag("option");
				writer.WriteLine();
			}
		}
		#endregion
	}

	[ToolboxData("<{0}:AxoDropDown runat=server></{0}:AxoDropDown>")]
	public class AxoDropDownList : System.Web.UI.WebControls.DropDownList
	{
		public virtual void SelectItemByValue(string itemValue)
		{
			ListItem item = Items.FindByValue(itemValue);
			if (item != null)
			{
				if (this.SelectedItem != null)
					this.SelectedItem.Selected = false;

				item.Selected = true;
			}
		}

		public void SelectItemByText(string itemText)
		{
			ListItem item = Items.FindByText(itemText);
			if (item != null)
			{
				if (this.SelectedItem != null)
					this.SelectedItem.Selected = false;

				item.Selected = true;
			}
		}

		public AxoDropDownList()
		{

		}
	}
}
