﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Text;

using PureChat.Plugins;

namespace PureChat.Plugins.OnTime
{
	public class OnTimePlugin : IPureChatPlugin
	{
		private Settings _settings;

		public System.Web.UI.UserControl LoadPluginControl(System.Web.UI.Page page, IChatObject chat, string pluginAction, string operatorName)
		{
			System.Web.UI.Control c = null;
			switch (pluginAction)
			{
				case "AddDefect":
					c = page.LoadControl("~/plugins/OnTime/AddDefect.ascx");
					break;
				case "AddFeature":
					c = page.LoadControl("~/plugins/OnTime/AddFeature.ascx");
					break;
				case "AddTask":
					c = page.LoadControl("~/plugins/OnTime/AddTask.ascx");
					break;
				case "AddIncident":
					c = page.LoadControl("~/plugins/OnTime/AddIncident.ascx");
					break;
			}

			if (c != null)
			{
				OnTimePluginControl plugin = (OnTimePluginControl)c;
				plugin.Settings = _settings;
				plugin.ChatObject = chat;
				plugin.OperatorName = operatorName;
			}
			
			return c as System.Web.UI.UserControl;
		}

		public string GetStyleSheetPath()
		{
			return "~/plugins/OnTime/OnTime.css";
		}

		private Dictionary<string, string> _pluginMethods;

		public Dictionary<string, string> GetPluginMethods()
		{
			if (_pluginMethods == null)
			{
				_pluginMethods = new Dictionary<string, string>(4);
				_pluginMethods["AddDefect"] = "Add Defect";
				_pluginMethods["AddFeature"] = "Add Feature";
				_pluginMethods["AddTask"] = "Add Task";
				_pluginMethods["AddIncident"] = "Add Incident";
			}

			return _pluginMethods;
		}

		public string GetPluginName()
		{
			return "OnTime";
		}

		public bool ConfigurePlugin()
		{
			try
			{
				_settings = new Settings(System.Web.HttpContext.Current.Server.MapPath("~/Plugins/OnTime/OnTimePlugin.config"));
			}
			catch
			{
				return false;
			}
			OnTimePluginService.Settings = _settings;
			return true;
		}
	}

	public class OnTimePluginControl : System.Web.UI.UserControl
	{
		public IChatObject ChatObject;
		public string OperatorName;
		public Settings Settings;

		protected string GetItemIdUrl(string itemId, string itemType)
		{
			if(string.IsNullOrEmpty(Settings.OnTimeWebUrl))
				return itemId;

			string result = itemId;
			switch (itemType)
			{
				case "defect":
					result = string.Format("<a target=\"_blank\" onclick=\"pureChat_OpenWindow({{0}}{0}Defects/ViewDefect.aspx?defectid={1}{{0}}); return false;\" href=\"{0}Defects/ViewDefect.aspx?defectid={1}\">{1}</a>", Settings.OnTimeWebUrl, itemId);
					break;
				case "feature":
					result = string.Format("<a target=\"_blank\" onclick=\"pureChat_OpenWindow({{0}}{0}Features/ViewFeature.aspx?featureid={1}{{0}}); return false;\" href=\"{0}Features/ViewFeature.aspx?featureid={1}\">{1}</a>", Settings.OnTimeWebUrl, itemId);
					break;
				case "task":
					result = string.Format("<a target=\"_blank\" onclick=\"pureChat_OpenWindow({{0}}{0}Tasks/ViewTask.aspx?taskid={1}{{0}}); return false;\" href=\"{0}Tasks/ViewTask.aspx?taskid={1}\">{1}</a>", Settings.OnTimeWebUrl, itemId);
					break;
				case "incident":
					result = string.Format("<a target=\"_blank\" onclick=\"pureChat_OpenWindow({{0}}{0}Incidents/ViewIncident.aspx?incidentnumber={1}{{0}}); return false;\" href=\"{0}Incidents/ViewIncident.aspx?incidentnumber={1}\">{1}</a>", Settings.OnTimeWebUrl, itemId);
					break;
			}

			return result.Replace("'", "\\'");
		}

		protected void BindDDL(DataView dv, System.Web.UI.WebControls.DropDownList ddl, string nameField, string valueField, string rootItemText, int rootItemValue)
		{
			ddl.Items.Clear();
			ddl.DataSource = dv;
			ddl.DataTextField = nameField;
			ddl.DataValueField = valueField;
			ddl.DataBind();
			if (rootItemText.Length > 0)
				ddl.Items.Insert(0, new System.Web.UI.WebControls.ListItem(rootItemText, rootItemValue.ToString()));
		}

		protected void SelectListItemByValue(System.Web.UI.WebControls.ListControl list, string itemValue)
		{
			if (list.SelectedItem != null)
				list.SelectedItem.Selected = false;

			if (list.Items.FindByValue(itemValue) != null)
				list.Items.FindByValue(itemValue).Selected = true;
		}

		protected void SelectListItemByValue(System.Web.UI.WebControls.ListControl list, int itemValue)
		{
			SelectListItemByValue(list, itemValue.ToString());
		}

		private readonly string[] DefaultValueTokens = { "[%Operator%]", "[%User%]", "[%Subject%]" };

		protected string GetDefaultValue(string defaultValue)
		{
			if(string.IsNullOrEmpty(defaultValue))
				return string.Empty;

			StringBuilder sb = new StringBuilder(defaultValue);
			foreach (string s in DefaultValueTokens)
			{
				int lastIndex = defaultValue.LastIndexOf(s, StringComparison.OrdinalIgnoreCase);
				while (lastIndex > -1)
				{
					sb.Remove(lastIndex, s.Length);
					switch (s)
					{
						case "[%Operator%]":
							sb.Insert(lastIndex, OperatorName);
							break;
						case "[%User%]":
							sb.Insert(lastIndex, ChatObject.RequestorName);
							break;
						case "[%Subject%]":
							sb.Insert(lastIndex, ChatObject.ChatSubject);
							break;
					}
					lastIndex = defaultValue.LastIndexOf(s, lastIndex, StringComparison.OrdinalIgnoreCase);
				}

				defaultValue = sb.ToString();
			}

			return defaultValue;
		}

		protected void GetUserNames(System.Web.UI.WebControls.DropDownList userList, string defaultValue)
		{
			using (OnTimeSDK.User.UserHandler handler = new PureChat.Plugins.OnTimeSDK.User.UserHandler(Settings.UserServiceUrl))
			{
				using (DataSet ds = handler.GetAllUsers(Settings.SecurityGuid))
				{
					ds.Tables[0].Columns.Add("Name", typeof(string));
					ds.Tables[0].Columns["Name"].Expression = "FirstName + ' ' + LastName";
					ds.Tables[0].DefaultView.RowFilter = "IsActive = 1";
					BindDDL(ds.Tables[0].DefaultView, userList, "Name", "UserId", " ", 0);
					if (!string.IsNullOrEmpty(defaultValue))
					{
						DataRow[] drs = ds.Tables[0].Select(string.Format("IsActive = 1 AND Name LIKE '%{0}%'", GetDefaultValue(defaultValue)));
						if (drs.Length > 0)
							SelectListItemByValue(userList, (int)drs[0]["UserId"]);
					}
					else
						userList.SelectedIndex = 0;
				}
			}
		}

		protected void GetProjects(HierarchyDropDown projectList, string defaultValue)
		{
			using (OnTimeSDK.Project.ProjectHandler handler = new PureChat.Plugins.OnTimeSDK.Project.ProjectHandler(Settings.ProjectServiceUrl))
			{
				using (DataSet ds = handler.GetAllProjects(Settings.SecurityGuid))
				{
					ds.Tables[0].DefaultView.RowFilter = "IsActive = 1";
					using (DataTable dt = CopySubTable(ds.Tables[0], "IsActive = 1", string.Empty))
					{
						projectList.DataTableSource = dt;
						projectList.RootProjectId = 0;
						projectList.ShowRootItem = true;
						projectList.RootItemText = "All Projects";
						projectList.DataTextField = "Name";
						projectList.DataIdField = "ProjectId";
						projectList.DataParentField = "ParentId";
						projectList.DataBind();
						if (!string.IsNullOrEmpty(defaultValue))
						{
							DataRow[] drs = ds.Tables[0].Select(string.Format("IsActive = 1 AND Name LIKE '%{0}%'", GetDefaultValue(defaultValue)));
							if (drs.Length > 0)
								projectList.SelectItemByValue((int)drs[0]["ProjectId"]);
						}
						else
							projectList.SelectedIndex = 0;
					}
					
				}
			}
		}

		protected void GetStatusTypes(System.Web.UI.WebControls.DropDownList statusList, string defaultValue)
		{
			using (OnTimeSDK.Types.TypeHandler handler = new PureChat.Plugins.OnTimeSDK.Types.TypeHandler(Settings.TypeServiceUrl))
			{
				using (DataSet ds = handler.GetStatusTypes(Settings.SecurityGuid))
				{
					BindDDL(ds.Tables[0].DefaultView, statusList, "Name", "StatusTypeId", " ", 0);
					if (!string.IsNullOrEmpty(defaultValue))
					{
						DataRow[] drs = ds.Tables[0].Select(string.Format("Name LIKE '%{0}%'", GetDefaultValue(defaultValue)));
						if (drs.Length > 0)
							SelectListItemByValue(statusList, (int)drs[0]["StatusTypeId"]);
					}
					else
						statusList.SelectedIndex = 0;
				}
			}
		}

		private static DataTable CopySubTable(DataTable table, string filter, string sort)
		{
			DataTable newTable = new DataTable();
			List<DataColumn> primaryKeyArray = new List<DataColumn>();

			// get primary keys
			foreach (DataColumn column in table.Columns)
			{
				DataColumn newColumn = newTable.Columns.Add(column.ColumnName, column.DataType, column.Expression);

				foreach (DataColumn keyColumn in table.PrimaryKey)
					if (column == keyColumn)
						primaryKeyArray.Add(newColumn);
			}
			newTable.PrimaryKey = primaryKeyArray.ToArray();

			// create a view instead of using Table.Select because of the bug in it (see Dan's blog)
			using (DataView dv = new DataView(table, filter, sort, DataViewRowState.CurrentRows))
			{
				foreach (DataRowView row in dv)
					newTable.ImportRow(row.Row);
			}

			return newTable;
		}
	}
}
